/* Copyright (c) 2006 Dirk Jagdmann <doj@cubic.org>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you
       must not claim that you wrote the original software. If you use
       this software in a product, an acknowledgment in the product
       documentation would be appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and
       must not be misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
       distribution. */

/* $Header: /code/cbmfs/d64.hpp,v 1.4 2006/07/21 14:46:04 doj Exp $ */

#ifndef D64__HPP
#define D64__HPP

#include "cbmdisk.hpp"

class D64 : public CBMdisk
{
public:

#pragma pack(push, 1)
  struct bam_64 {
    /*0x00*/ TS dir;
    /*0x02*/ BYTE version;		/* should be 0x41 */
    /*0x03*/ BYTE onesided;
    /*0x04*/ bamentry entry[35];
    /*0x90*/ BYTE diskname[16];
    /*0xA0*/ BYTE unused1[2];		/* should be 0xA0 */
    /*0xA2*/ BYTE id[2];
    /*0xA4*/ BYTE unused2;		/* should be 0xA0 */
    /*0xA5*/ BYTE DOStype[2];		/* should be "2A" */
    /*0xA7*/ BYTE unused3[4];		/* should be 0xA0 */
    /*0xAB*/ BYTE unused4[0x100-0xAB];	/* should be 0 */
  };
  BOOST_STATIC_ASSERT(sizeof(bam_64) == SECTOR_SIZE);
#pragma pack(pop)

  D64(BYTE *disk_);
  virtual TS dir();
  virtual int maxblocks();
  virtual void bam_set(const BYTE t, const BYTE s, const bool b);
  virtual void printbam();
  virtual int track_max();

protected:
  virtual const int* sectorsPerTrack();
  virtual bool bam_get(const BYTE t, const BYTE s);
  virtual void fix(bamentry*, const BYTE t);
  virtual bamentry* getbamentry(const BYTE t);
  virtual void printbaminfo();
  virtual bool reserved_track(const BYTE t);

private:
  D64();
  D64(const D64&);
  D64& operator= (const D64&);
};

#endif
